@@ -75,12 +75,14 @@ katze_item_class_init (KatzeItemClass* class)
      *
      * Emitted when a meta data value was changed.
      *
+     * Since 0.2.2 details according to keys are supported.
+     *
      * Since: 0.1.9
      */
     signals[META_DATA_CHANGED] = g_signal_new (
         "meta-data-changed",
         G_TYPE_FROM_CLASS (class),
-        (GSignalFlags)(G_SIGNAL_RUN_LAST),
+        (GSignalFlags)(G_SIGNAL_RUN_LAST |G_SIGNAL_DETAILED),
         0,
         0,
         NULL,
@@ -187,7 +189,6 @@ katze_item_finalize (GObject* object)
     g_free (item->name);
     g_free (item->text);
     g_free (item->uri);
-    g_free (item->icon);
     g_free (item->token);
 
     g_hash_table_unref (item->metadata);
@@ -215,7 +216,7 @@ katze_item_set_property (GObject*      object,
         katze_assign (item->uri, g_value_dup_string (value));
         break;
     case PROP_ICON:
-        katze_assign (item->icon, g_value_dup_string (value));
+        katze_item_set_icon (item, g_value_get_string (value));
         break;
     case PROP_TOKEN:
         katze_assign (item->token, g_value_dup_string (value));
@@ -252,7 +253,7 @@ katze_item_get_property (GObject*    object,
         g_value_set_string (value, item->uri);
         break;
     case PROP_ICON:
-        g_value_set_string (value, item->icon);
+        g_value_set_string (value, katze_item_get_icon (item));
         break;
     case PROP_TOKEN:
         g_value_set_string (value, item->token);
@@ -396,7 +397,7 @@ katze_item_get_icon (KatzeItem* item)
 {
     g_return_val_if_fail (KATZE_IS_ITEM (item), NULL);
 
-    return item->icon;
+    return katze_item_get_meta_string (item, "icon");
 }
 
 /**
@@ -412,7 +413,7 @@ katze_item_set_icon (KatzeItem*   item,
 {
     g_return_if_fail (KATZE_IS_ITEM (item));
 
-    katze_assign (item->icon, g_strdup (icon));
+    katze_item_set_meta_string (item, "icon", icon);
     g_object_notify (G_OBJECT (item), "icon");
 }
 
@@ -510,7 +511,7 @@ katze_item_set_meta_data_value (KatzeItem*   item,
         g_hash_table_insert (item->metadata, g_strdup (&key[7]), value);
     else
         g_hash_table_insert (item->metadata, g_strdup (key), value);
-    g_signal_emit (item, signals[META_DATA_CHANGED], 0, key);
+    g_signal_emit (item, signals[META_DATA_CHANGED], g_quark_from_string (key), key);
 }
 
 /**
@@ -590,7 +591,7 @@ katze_item_get_meta_integer (KatzeItem*   item,
     if (g_str_has_prefix (key, "midori:"))
         key = &key[7];
     if (g_hash_table_lookup_extended (item->metadata, key, NULL, &value))
-        return g_ascii_strtoll (value, NULL, 0);
+        return value ? g_ascii_strtoll (value, NULL, 0) : -1;
     return -1;
 }
 
@@ -696,7 +697,6 @@ katze_item_copy (KatzeItem* item)
         "name", item->name,
         "text", item->text,
         "uri", item->uri,
-        "icon", item->icon,
         "token", item->token,
         "added", item->added,
         "parent", item->parent,
