@@ -18,6 +18,7 @@
 #endif
 
 #include <katze/katze.h>
+#include "midori-view.h"
 
 G_BEGIN_DECLS
 
@@ -75,7 +76,7 @@ struct _MidoriBrowserClass
 };
 
 GType
-midori_browser_get_type               (void);
+midori_browser_get_type               (void) G_GNUC_CONST;
 
 MidoriBrowser*
 midori_browser_new                    (void);
@@ -122,6 +123,10 @@ midori_browser_set_current_page       (MidoriBrowser*     browser,
 gint
 midori_browser_get_current_page       (MidoriBrowser*     browser);
 
+GtkWidget*
+midori_browser_get_nth_tab            (MidoriBrowser*     browser,
+                                       gint               n);
+
 void
 midori_browser_set_current_tab        (MidoriBrowser*     browser,
                                        GtkWidget*         widget);
@@ -129,6 +134,12 @@ midori_browser_set_current_tab        (MidoriBrowser*     browser,
 GtkWidget*
 midori_browser_get_current_tab        (MidoriBrowser*     browser);
 
+GList*
+midori_browser_get_tabs               (MidoriBrowser*     browser);
+
+KatzeArray*
+midori_browser_get_proxy_items        (MidoriBrowser*     browser);
+
 KatzeArray*
 midori_browser_get_proxy_array        (MidoriBrowser*     browser);
 
@@ -141,6 +152,9 @@ midori_browser_quit                   (MidoriBrowser*     browser);
 const gchar**
 midori_browser_get_toolbar_actions    (MidoriBrowser*     browser);
 
+MidoriWebSettings*
+midori_browser_get_settings           (MidoriBrowser*     browser);
+
 G_END_DECLS
 
 #endif /* __MIDORI_BROWSER_H__ */
